// Source               ::  AL5D_V1_Calibrate

// Description          ::  Basic Robotic Arm calibration program.
//                          Originally designed for Lynxmotion AL5D, but can be used with any robotic arm,
//                          just modify the number of servos to match your arm. 
//
//                          Use this to experiment with arm angles and algotithms.
//
//                          Performs best on my arm.  Make an appointment :)   ...
//                          If your servos are not configured exactly the same as mine, you might have to swap
//                          angles in various spots ( ie 180 instead of 0, etc ).  You'll know that this is necessary
//                          if this program causes some of your servos to run "opposite" to what you expect.


//                          Recommended components ( or equivalent ) 

//                          1)  Lynxmotion AL5D 4 Degrees of Freedom (4DoF) Robotic Arm Combo Kit (BotBoarduino)
//                                   http://www.robotshop.com/en/lynxmotion-al5d-4-degrees-freedom-robotic-arm-combo-kit.html 
//                          2)  Lynxmotion Wrist Rotate Upgrade (Heavy Duty)
//                                   http://www.robotshop.com/en/lynxmotion-heavy-duty-wrist-rotate-upgrade.html


// Original Program     ::  Robotic Arm Calibration Program     - Joe M  / arduino (at) bellaliant.net   / May 2016
//                          

// Modified By          ::  Your Name Here / Contact / Date  



//  This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
//  as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  Lesser General Public License for more details.
//
//  For a copy of the GNU Lesser General Public License, 
//  write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


// **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T ****
// ***********************************************************************************************************************************************************************
// Pinouts... IMPORTANT!!!! ... set the pinouts to your hardware configuration
//
// This program uses the following components.  Constants are included for your own expansion.
//
// ***********************************************************************************************************************************************************************

// ********** Digital Pins D0 thru D13 **********

// Using 0 and 1 disables upload!     

#define DO_NOT_USE_0         0
#define DO_NOT_USE_1         1

#define SHOULDER_PIN         2
#define ELBOW_PIN            3
#define WRIST_PIN            4
        
#define NOT_USED_5           5

#define BASE_PIN             6
#define NOT_USED_7           7
#define NOT_USED_8           8
#define NOT_USED_9           9
#define WRISTROTATE_PIN     10 
#define GRIPPER_PIN         11
#define NOT_USED_12         12  
#define NOT_USED_13         13  



// ********** Analog Pins A0 thru A5 **********

// FYI... using analog pins for digital I/O is just the same as using digital ones.

// A0 is referred to as Pin 14     A1 is referred to as Pin 15
// A2 is referred to as Pin 16     A3 is referred to as Pin 17
// A4 is referred to as Pin 18     A5 is referred to as Pin 19

#define NOT_USED_A0         A0 
#define NOT_USED_A1         A1
#define NOT_USED_A2         A2 
#define NOT_USED_A3         A3 
#define NOT_USED_A4         A4  
#define NOT_USED_A5         A5       





// *********************************************************************************************************************************************************************
// Required includes <--------------------------------------------------------------------------------------------------------------------------------------------------
// *********************************************************************************************************************************************************************

#include <Servo.h> 



// *********************************************************************************************************************************************************************
//  Global Variables.
//  Professional programmers cringe at the thought of globals, but they seem to be generally acceptable in the "wild west" Arduino world.
//  Prudent use of them simplifies program design since variable passing via parameters between functions are reduced.
// *********************************************************************************************************************************************************************

// An homage to every program I've written since my WATFIV days.
// Every program needs an "int i", right?
int i;

// Servos that make up the AL5D
Servo Base, Shoulder, Elbow, Wrist, WristRotate, Gripper;

// Store the current servo angles
int   currentBaseAngle, currentShoulderAngle, currentElbowAngle, 
      currentWristAngle, currentWristRotateAngle, currentGripperAngle;


// Control the speed of the arm movements.  Increase angle, decrease delay to make arm react faster.
int   servoStepAngle = 2 , servoStepDelay = 25;




// **********************************************************************************************************************************************************************
// Function Prototypes <-------------------------------------------------------------------------------------------------------------------------------------------------                                                                                                                                               
// **********************************************************************************************************************************************************************


void rotateBase           ( const int& endAt );
void rotateShoulder       ( const int& endAt );
void rotateElbow          ( const int& endAt );
void rotateWrist          ( const int& endAt );
void rotateWristRotate    ( const int& endAt );
void rotateGripper        ( const int& endAt );
void rotateServo          ( const int& endAt , Servo& whichServo , int& currentAngle );

void deployArms           ( void );
void grabVerticalItem     ( void );
void dropItem             ( void );
void autonomousDemo       ( void );


// Functions with default parameters
// http://en.cppreference.com/w/cpp/language/default_arguments

void extendArm            ( const int& = 34 , const bool& = 0 );    // If no arguments provided, default Shoulder height is 34, default "withItem" is no
void retractArm           ( const int& = 150 );                     // If no argument provided, default Shoulder height is 150


// *********************************************************************************************************************************************************************
// Main program <-------------------------------------------------------------------------------------------------------------------------------------------------------                                                                                                                                                                                                                   
// *********************************************************************************************************************************************************************
 
void setup() {
    
    deployArms( );

}

void loop() {

    // Put your experimentation code here...

    extendArm ( );

    delay(5000);

    retractArm ( );

    delay(5000);
    

}



// *********************************************************************************************************************************************************************
// Functions <---------------------------------------------------------------------------------------------------------------------------------------------------------- 
// *********************************************************************************************************************************************************************


// Incoming parameters :   constant reference to an int denoting servo angle to move to.  
// Global variables    :   Uses    -   "Servo", current"Servo"Angle
//                         Changes -  
// Responsibilities    :   Passes servo variables on to generic servo control function.
// Returns             :   Nothing.

void rotateBase         ( const int& endAt ) {  rotateServo( endAt , Base ,        currentBaseAngle );        }
void rotateShoulder     ( const int& endAt ) {  rotateServo( endAt , Shoulder ,    currentShoulderAngle );    }
void rotateElbow        ( const int& endAt ) {  rotateServo( endAt , Elbow ,       currentElbowAngle );       }
void rotateWrist        ( const int& endAt ) {  rotateServo( endAt , Wrist ,       currentWristAngle );       }
void rotateWristRotate  ( const int& endAt ) {  rotateServo( endAt , WristRotate , currentWristRotateAngle ); }
void rotateGripper      ( const int& endAt ) {  rotateServo( endAt , Gripper ,     currentGripperAngle );     }


void rotateServo ( const int& endAt , Servo& whichServo , int& currentAngle ) {  

    // Incoming parameters :   constant reference to an int denoting servo angle to move to,
    //                         reference to a Servo instance,
    //                         reference to the current angle of the Servo
      
    // Global variables    :   Uses    -   servoStepAngle, servoStepSpeed
    //                         Changes -   "current...Angle" of Servo passed by reference
    
    // Responsibilities    :   Moves desired Servo to requested angle.  Changes "current...Angle" of that Servo.
    //                         Why this function?  Moving a Servo from one angle to another can be a rather fast and brutal action,
    //                         not very conducive to finely controlling the action(s) of the arm.
    //                         This function slows the movements down to a smooth and managable speed.
    //                         Want faster or slower?   Just manipulate the values of globals servoStepAngle and servoStepSpeed.
    
    // Returns             :   Nothing.
    
    if ( currentAngle == endAt ) return;

    if ( currentAngle < 0 ) { 
      tone(5, 999, 250);  // Stop that!
      whichServo.write(currentAngle = 2);    delay(servoStepDelay);
      return;
    }

    if ( currentAngle > 180 ) { 
      tone(5, 999, 250);  // Stop that!
      whichServo.write(currentAngle = 178);    delay(servoStepDelay);
      return;
    }

    
    if ( endAt > currentAngle ) {
        for ( int i = currentAngle; i <= endAt; i += servoStepAngle ) { 
            
            whichServo.write(currentAngle = i);    delay(servoStepDelay); 
            
        }
    }
    else {
        for ( int i = currentAngle; i >= endAt; i -= servoStepAngle ) { 
                        
            whichServo.write(currentAngle = i);    delay(servoStepDelay); 
              
        }
    }
   
}


void deployArms ( void ) {

    // Incoming parameters :   Nothing.
      
    // Global variables    :   Uses    -   
    //                         Changes -  currentBaseAngle, currentShoulderAngle, currentElbowAngle, 
    //                                    currentWristAngle, currentWristRotateAngle, currentGripperAngle,
    //                                    currentBackElbowAngle, currentBackGripperAngle
    
    // Responsibilities    :   Set arm(s) and global current...Angle's to initial working position. 
    //                         You can fine tune this to other behaviour if you wish.
    
    // Returns             :   Nothing.


    delay( 500);   Shoulder.attach     (SHOULDER_PIN);      Shoulder.write    (currentShoulderAngle     =  90); 
    delay( 500);   Elbow.attach        (ELBOW_PIN);         Elbow.write       (currentElbowAngle        = 136); 
    
    delay( 500);   Wrist.attach        (WRIST_PIN);         Wrist.write       (currentWristAngle        =  70);
    delay( 500);   WristRotate.attach  (WRISTROTATE_PIN);   WristRotate.write (currentWristRotateAngle  = 160);
    delay( 500);   Gripper.attach      (GRIPPER_PIN);       Gripper.write     (currentGripperAngle      = 180);                                                        
    
    delay( 500);   Base.attach         (BASE_PIN);          Base.write        (currentBaseAngle         =  90);
                                                            
    delay(1000);   retractArm( ); 
 
}



void retractArm ( const int& shoulderStop ) {

    // Incoming parameters :   constant reference to an int denoting ending angle of Shoulder servo.
      
    // Global variables    :   Uses    -  servoStepAngle, servoStepDelay 
    //                         Changes -  currentShoulderAngle, currentElbowAngle, currentWristAngle
    
    // Responsibilities    :   Retracts the arm in a predetermined manner.  
    //                         A function of convenience, takes the place of the user having to perform
    //                         a number of actions.   Modify to suit your needs. 
    
    // Returns             :   Nothing.
    
    int braking = 0;
    
    for ( int i =  currentShoulderAngle; i <=  shoulderStop; i += servoStepAngle ) { 
        
        Shoulder.write(currentShoulderAngle = i);
        
        if ( (i+86) <= 180 ) Elbow.write(currentElbowAngle = i+86);
        
        // Keep the wrist as horizontal as possible
        if      ( i <= 90  ) Wrist.write(currentWristAngle = 140);
        else if ( i >= 150 ) Wrist.write(currentWristAngle =  75);             
        else                 Wrist.write(currentWristAngle = ( 140 - ( i - 90 )) ); 
        
        // As you rise towards 90 degrees, weight is working for you, keeping things slow.
        // Past 90 degrees, weight works against you as it increases the speed of the arm,
        // resulting in an momentum that builds the farther you go.
        // The following formula calculates an extra time in servo delay, resulting in a
        // braking action.
        braking = ( i > 90 ) ?  ( ( i - 90 ) / 10 ) * 15  : 0;
        delay(servoStepDelay+braking); 
        
    }

  
    delay(1000);
  
}

void extendArm ( const int& shoulderStop , const bool& withItem ) {

    // Incoming parameters :   constant reference to an int denoting ending angle of Shoulder servo.
    //                         constant reference to a bool denoting if the arm is holding an item ( 0 - no , 1 - yes ).
      
    // Global variables    :   Uses    -  servoStepAngle, servoStepDelay 
    //                         Changes -  currentShoulderAngle, currentElbowAngle, currentWristAngle
    
    // Responsibilities    :   Extend the arm in a predetermined manner.  
    //                         A function of convenience, takes the place of the user having to perform
    //                         a number of actions.   Modify to suit your needs.
    
    // Returns             :   Nothing.
   

    for ( int i =  currentShoulderAngle; i >=  shoulderStop; i -= servoStepAngle ) { 
        
        if ( (i+86) <= 180 ) Elbow.write(currentElbowAngle = i+86);  
        
        Shoulder.write(currentShoulderAngle = i);     
        
        if ( withItem ) {
            // Keep the wrist as horizontal as possible
            if      ( i <= 90  ) Wrist.write(currentWristAngle = 140);
            else if ( i >= 150 ) Wrist.write(currentWristAngle =  75);             
            else                 Wrist.write(currentWristAngle = ( 140 - ( i - 90 )) );
        }
        else {
            // Move the wrist to angle up onto an item that is to be grasped
                                 Wrist.write(currentWristAngle = 160-i);
        }
        
        delay(servoStepDelay); 
    }


    
    delay(1000);
  
}



void grabVerticalItem ( void ) {
 
    // Incoming parameters :   Nothing.
      
    // Global variables    :   Uses    -   
    //                         Changes -  
    
    // Responsibilities    :   Grabs a vertical item (something "standing up" rather than "laying down") with the Gripper  
    //                         A function of convenience, takes the place of the user having to perform
    //                         a number of actions.   Modify to suit your needs.
    
    // Returns             :   Nothing.
      
    delay( 100);   rotateWristRotate  ( 70);  // Rotate to grab a vertical item ( ie | )
    delay( 100);   rotateGripper      ( 00);  // Open 
    delay(2000);   rotateGripper      (180);  // Close
    //delay(2000);   rotateWristRotate  (160);  // Rotate item to horizontal ( ie --- )
    //delay( 100);   rotateWrist        (180);  // Raise
    //delay(1000);
  
}

void dropItem ( void ) {

    // Incoming parameters :   Nothing.
      
    // Global variables    :   Uses    -  
    //                         Changes -  
    
    // Responsibilities    :   Releases an item from the Gripper.  
    //                         A function of convenience, takes the place of the user having to perform
    //                         a number of actions.   Modify to suit your needs.  
    
    // Returns             :   Nothing.
    
   
    delay( 500);   rotateWrist    ( 40);   // Lower  
    delay( 500);   rotateGripper  ( 00);   // Open 
    delay( 500);   rotateShoulder ( 44);   // Raise arm a little
    delay( 500);   rotateGripper  (180);   // Close
    delay( 500);   rotateWrist    (180);   // Raise 
    delay( 500);
  
}





void autonomousDemo ( void ) {

    // Incoming parameters :   Nothing.
      
    // Global variables    :   Uses    -   
    //                         Changes -  currentBaseAngle
    
    // Responsibilities    :   Run the arm through a series of actions for demonstration purposes.
    
    // Returns             :   Nothing.
    

    // Grab an item from the front
    rotateBase        ( 90);    // Move arm to front
    rotateWristRotate ( 70);    // Rotate to grab a vertical item ( ie | )
    rotateGripper     ( 00);    // Open before moving arm into position                
    extendArm         ( 14);    // Modify to suit your height requirement   
    rotateWrist       (130);    // Tweak wrist angle if required  
    rotateGripper     (180);    // Close on item     
    retractArm        (110);    // Move back to this Shoulder angle
  
    delay(1000);
    
    // Place it to the right
    rotateBase        (180);    // Move arm full right
    rotateWristRotate ( 70);    // Rotate to place vertical item ( ie | )           
    extendArm         (14,1);   // Modify to suit your height requirement
    //rotateWrist       (???);    // Tweak wrist angle if required    
    rotateGripper     ( 00);    // Open to release item     
    retractArm        (   );    // Park arm
    
    
    // Grab an item from the right
    rotateBase        (180);    // Move arm full right
    rotateWristRotate ( 70);    // Rotate to grab a vertical item ( ie | )
    rotateGripper     ( 00);    // Open before moving arm into position                
    extendArm         ( 14);    // Modify to suit your height requirement 
    //rotateWrist       (???);    // Tweak wrist angle if required
    rotateGripper     (180);    // Close on item      
    retractArm        (110);    // Move back to this Shoulder angle

    delay(1000);
    
    // Place it in front
    rotateBase        ( 90);    // Move arm to front
    rotateWristRotate ( 70);    // Rotate to place vertical item ( ie | )           
    extendArm         (14,1);   // Modify to suit your height requirement
    rotateWrist       (130);    // Tweak wrist angle if required    
    rotateGripper     ( 00);    // Open to release item      
    retractArm        (   );    // Park arm
    
    
}


